#ifndef __MAIN_H
#define __MAIN_H

#include "ivm.h"

#define CHECK_RESULT(Result, Device)                                                        \
{                                                                                           \
    if (Result != result_ok)                                                                \
    {                                                                                       \
        printf("\n");                                                                       \
        if (Result == result_error)                                                         \
            printf("Runtime error in %s:%d\n", __FILE__, __LINE__);                         \
        else if (Result == result_not_implemented)                                          \
            printf("Command not implemented error in %s:%d\n", __FILE__, __LINE__);         \
        else if (Result == result_value_error)                                              \
            printf("Value error in %s:%d\n", __FILE__, __LINE__);                           \
        else if (Result == result_nodevice)                                                 \
            printf("No device (connection loss) error in %s:%d\n", __FILE__, __LINE__);     \
                                                                                            \
        ivm_close_device(&Device);                                                          \
        return;                                                                             \
    }                                                                                       \
}                                                                                           \

#ifdef _WIN32
//  For Windows (32- and 64-bit)
#include <windows.h>
#define SLEEP_MS(msecs) Sleep(msecs)
#elif __unix
//  For linux, OSX, and other unixes
#define _POSIX_C_SOURCE 199309L // or greater
#include <time.h>
#define SLEEP_MS(msecs) do {            \
struct timespec ts;                     \
    ts.tv_sec = msecs / 1000;           \
    ts.tv_nsec = msecs % 1000 * 1000;   \
    nanosleep(&ts, NULL);               \
} while (0)
#else
#   error "Unknown system"
#endif


#define ARRAY_LEN(array) (sizeof(array) / sizeof(array[0]))


#ifdef _WIN32
//  For Windows (32- and 64-bit)
#include <conio.h>
#define WAIT_FOR_ANY_KEY() getch()
#elif __unix
//  For linux, OSX, and other unixes
static inline void getch()
{
  fflush(stdout);
  char ch;
  scanf("%c ",&ch);
}
#define WAIT_FOR_ANY_KEY() getch()
#else
#   error "Unknown system"
#endif

#define NUM_POINTS_IN_CURVE 100

typedef struct
{
    float Voltage[NUM_POINTS_IN_CURVE]; // in Volts
    float Current[NUM_POINTS_IN_CURVE]; // in mA
} iv_curve_t;

device_t OpenDevice(void);
void PrintDeviceInfo(device_t Device);
void CalibrateDevice(device_t Device);
void SetupDevice(device_t Device);
void MakeMeasurement(device_t Device, iv_curve_t *IVCurve);
void SaveMeasurement(iv_curve_t *IVCurve);
void CloseDevice(device_t Device);

#endif // __MAIN_H
